--BINDING_HEADER_WOWQUOTE_HEADER = "WoWQuote"
--BINDING_NAME_WOWQUOTE_TOGGLE = "Toggle WoWQuote"

WQUI = {
  NUM_DISPLAYABLE = 19,  -- maxmimum number of quotes that are visible at a time

  CategoryDropDown = {}, -- hold functions for the Category DropDown
  ChannelDropDown = {},  -- hold functions for the Channel DropDown
  DisplayedItems = {},   -- all quotes displayed for the current tag and search pattern
  FilteredItems = {},    -- all quotes displayed for the current tag

  Quote = WQ2_guild,      -- the quote function that is used for posting quotes
  Channels = {
    { name=WQ2_channels.PARTY.name, func=WQ2_party },
    { name=WQ2_channels.GUILD.name, func=WQ2_guild },
    { name=WQ2_channels.RAID.name, func=WQ2_raid },
    { name=WoWQuote2UI_Localization.TEST_LISTEN_CHANNEL, func=WQ2_listen },
  },

  Sorters = {
    ByMessage = function(a, b) return (a.msg < b.msg) end,
    ByID = function(a, b) return (a.id < b.id) end
  },
  Sorter = function(a, b) return (a.msg < b.msg) end,

}


function WQUI:Msg(msg)
  DEFAULT_CHAT_FRAME:AddMessage(msg, 1.0, 0.5, 0.5)
end

function WQUI:Toggle()
  if (WQUI:IsVisible()) then
    HideUIPanel(WQUI)
  else
    ShowUIPanel(WQUI)
  end
end

function WQUI:Localize()
  local L = WoWQuote2UI_Localization
  local name = self:GetName()

  getglobal(name.."Title"):SetText(L.DIALOG_TITLE)
  self.Columns.Text:SetText(L.COLUMN_TEXT)
  self.Columns.ID:SetText(L.COLUMN_DURATION)

  BINDING_HEADER_WOWQUOTE_HEADER = L.BINDING_HEADER
  BINDING_NAME_WOWQUOTE_TOGGLE = L.BINDING_TOGGLE

end

function WQ2_scrollBar() end

function WQUI:Initialize()
  UIPanelWindows[self:GetName()] = { area = "left", pushable = 5, whileDead = 1 };

  self:Localize()
  self.Items = {}

  -- create list items
  self.Items[1] = CreateFrame("Button", "WoWQuote2Item1", self, "WoWQuote2ItemTemplate")
  self.Items[1]:SetPoint("TOPLEFT", self, "TOPLEFT", 20, -120) --letzer parameter -100

  for i=2,self.NUM_DISPLAYABLE do
    self.Items[i] = CreateFrame("Button", "WoWQuote2Item"..i, self, "WoWQuote2ItemTemplate")
    self.Items[i]:SetPoint("TOPLEFT", self.Items[i-1], "BOTTOMLEFT", 0, 0)
  end

  self:UpdateCheckButtons()

  --self:SelectCategory(0, WQmedia) -- "all"
  --WQUI.FilteredItems = WQ2_getFilteredMedia(nil); --"all"
  --WQUI.DisplayedItems = WQUI.FilteredItems;
  --self:OnSearch();

end

function WQUI:UpdateItems()
  local numItems = # self.DisplayedItems;
  local scrollFrame = getglobal(self:GetName().."ScrollFrame")
  local offset = FauxScrollFrame_GetOffset(scrollFrame) or 0

  -- display quotes according to the scrollbar's offset
  for i=1,self.NUM_DISPLAYABLE do
    local button = self.Items[i]

    if (i <= numItems) then
      local media = self.DisplayedItems[i+offset]
      local buttonName = button:GetName()

      getglobal(buttonName.."LabelsText"):SetText(media.msg)
      getglobal(buttonName.."LabelsID"):SetText(media.id)
      button.id = media.id
      button:Show();
    else -- hide unused button if there are less quotes than buttons
      button:Hide();
    end
  end

  FauxScrollFrame_Update(scrollFrame, numItems, self.NUM_DISPLAYABLE, 16);
end

function WQUI:UpdateCheckButtons()
  self.CheckButtons.Guild:SetChecked(WQ2_SETTINGS.broadcast.GUILD);
  self.CheckButtons.Party:SetChecked(WQ2_SETTINGS.broadcast.PARTY);
  self.CheckButtons.Raid:SetChecked(WQ2_SETTINGS.broadcast.RAID);
end

function WQUI:SortItems(...)
  local arg = {...}

  if (arg[1]) then
    self.Sorter = arg[1]
  end
  sort(WQUI.DisplayedItems, self.Sorter)
end

function WQUI:OnSearch()

  --erst Tags filtern
  --evtl 2 methoden, sodass tags nur gefiltert werden, wenn sie sich verändern

  if self.EditBox:GetText() ~= "" and self.EditBox:GetText() ~= WoWQuote2UI_Localization.SEARCH  then
    --wenn Suchfester nicht leer und nicht default text: suchen
    local pattern = string.lower(self.EditBox:GetText());
    WQUI.DisplayedItems = {};

    for i,v in pairs(WQUI.FilteredItems) do
      if string.find(string.lower(v.msg), pattern) then
        table.insert(WQUI.DisplayedItems, v);
      end
    end
  else
    WQUI.DisplayedItems = WQUI.FilteredItems;
  end

  --WQ2_print("OnSearch: DisplayedItems length: "..#WQUI.DisplayedItems);

  self:SortItems();
  FauxScrollFrame_SetOffset(getglobal(self:GetName().."ScrollFrame"), 0);
  self:UpdateItems();

end

function WQUI.CategoryDropDown.OnClick(self)

  local tag = self.value;

  --WQ2_print("tag to filter: "..tostring(tag));


  WQUI.FilteredItems = WQ2_getFilteredMedia(WQUI.tagList[tag]);

  if not tag then
    tag = 0;
  end

  --WQ2_print("tag to show: "..tostring(tag));

  UIDropDownMenu_SetSelectedValue(WoWQuote2DialogCategoryDropDown, tag);

  WQUI:OnSearch();
end

function WQUI.CategoryDropDown.Initialize()
  UIDropDownMenu_AddButton( {text=ALL, value=0, func=WQUI.CategoryDropDown.OnClick} );

  WQUI.tagList={};

  for i,v in pairs(WQ2_Tags) do
    table.insert(WQUI.tagList, {i,v})
  end
  table.sort(WQUI.tagList, function(a,b) return a[2]<b[2] end);

  for i,tag in ipairs(WQUI.tagList) do
    UIDropDownMenu_AddButton( {text=tag[2], value=i, func=WQUI.CategoryDropDown.OnClick, owner=WoWQuote2Dialog:GetParent()} );
  end
end

function WQUI.ChannelDropDown.OnClick(self)
  local channel = self:GetID()

  UIDropDownMenu_SetSelectedID(WoWQuote2DialogChannelDropDown, channel);

  WQUI.Quote = WQUI.Channels[channel].func
end

function WQUI.ChannelDropDown.Initialize()
  for i=1, # WQUI.Channels do
    UIDropDownMenu_AddButton( {text=WQUI.Channels[i].name, func=WQUI.ChannelDropDown.OnClick, value=i} );
  end
end
